package Examples;

1;

=pod

=head1 EXAMPLES

=head2 EXAMPLE1

=for comment filename=first.pl

 use strict;
 use warnings;

 # Print.
 print "First.\n";

=head2 EXAMPLE2

=for comment filename=second.pl

 use strict;
 use warnings;

 # Print.
 print "Second.\n";

=cut
